@@ -55,6 +55,7 @@ init_request (OM_uint32 * minor_status,
   gss_buffer_desc tmp;
   Shishi_tkts_hint hint;
 
+  /* Get service ticket. */
   maj_stat = gss_krb5_canonicalize_name (minor_status, target_name,
 					 GSS_C_NO_OID, &k5->peerptr);
   if (GSS_ERROR (maj_stat))
@@ -62,6 +63,7 @@ init_request (OM_uint32 * minor_status,
 
   memset (&hint, 0, sizeof (hint));
   hint.server = k5->peerptr->value;
+  hint.endtime = time_req;
 
   k5->tkt = shishi_tkts_get (shishi_tkts_default (k5->sh), &hint);
   if (!k5->tkt)
@@ -71,6 +73,7 @@ init_request (OM_uint32 * minor_status,
       return GSS_S_NO_CRED;
     }
 
+  /* Create Authenticator checksum field. */
   maj_stat = _gss_krb5_checksum1964_pack (initiator_cred_handle,
 					  input_chan_bindings,
 					  req_flags,
@@ -78,12 +81,19 @@ init_request (OM_uint32 * minor_status,
   if (GSS_ERROR (maj_stat))
     return maj_stat;
 
+  /* Create AP-REQ in output_token. */
   rc = shishi_ap_tktoptionsraw (k5->sh, &k5->ap, k5->tkt,
 				SHISHI_APOPTIONS_MUTUAL_REQUIRED,
 				0x8003, cksum, cksumlen);
   if (rc != SHISHI_OK)
     return GSS_S_FAILURE;
 
+  rc = shishi_authenticator_seqnumber_get (k5->sh,
+					   shishi_ap_authenticator (k5->ap),
+					   &k5->initseqnr);
+  if (rc != SHISHI_OK)
+    return GSS_S_FAILURE;
+
   rc = shishi_ap_req_der (k5->ap, &tmp.value, &tmp.length);
   if (rc != SHISHI_OK)
     return GSS_S_FAILURE;
@@ -93,20 +103,8 @@ init_request (OM_uint32 * minor_status,
   if (!rc)
     return GSS_S_FAILURE;
 
-  k5->flags = req_flags & (/* GSS_C_DELEG_FLAG | */
-			   GSS_C_MUTUAL_FLAG |
-			   GSS_C_REPLAY_FLAG | GSS_C_SEQUENCE_FLAG |
-			   GSS_C_CONF_FLAG | GSS_C_INTEG_FLAG);
-
-  k5->key = shishi_ap_key (k5->ap);
-  k5->reqdone = 1;
-
   if (req_flags & GSS_C_MUTUAL_FLAG)
-    {
-      if (ret_flags)
-	*ret_flags |= GSS_C_MUTUAL_FLAG;
-      return GSS_S_CONTINUE_NEEDED;
-    }
+    return GSS_S_CONTINUE_NEEDED;
 
   return GSS_S_COMPLETE;
 }
@@ -130,8 +128,8 @@ init_reply (OM_uint32 * minor_status,
 {
   gss_ctx_id_t ctx = *context_handle;
   _gss_krb5_ctx_t k5 = ctx->krb5;
-  int rc;
   gss_buffer_desc data;
+  int rc;
 
   rc = gss_decapsulate_token_check (input_token, TOK_AP_REP, TOK_LEN,
 				    GSS_KRB5, &data);
@@ -139,6 +137,7 @@ init_reply (OM_uint32 * minor_status,
     return GSS_S_DEFECTIVE_TOKEN;
 
   rc = shishi_ap_rep_der_set (k5->ap, data.value, data.length);
+  gss_release_buffer (NULL, &data);
   if (rc != SHISHI_OK)
     return GSS_S_DEFECTIVE_TOKEN;
 
@@ -150,9 +149,7 @@ init_reply (OM_uint32 * minor_status,
 					  shishi_ap_encapreppart (k5->ap),
 					  &k5->acceptseqnr);
   if (rc != SHISHI_OK)
-    k5->acceptseqnr = 0;
-
-  k5->repdone = 1;
+    return GSS_S_DEFECTIVE_TOKEN;
 
   return GSS_S_COMPLETE;
 }
@@ -176,10 +173,16 @@ gss_krb5_init_sec_context (OM_uint32 * minor_status,
 {
   gss_ctx_id_t ctx = *context_handle;
   _gss_krb5_ctx_t k5 = ctx->krb5;
+  OM_uint32 maj_stat;
   int rc;
 
+  if (minor_status)
+    *minor_status = 0;
+
   if (ret_flags)
-    *ret_flags = 0;
+    *ret_flags = GSS_C_REPLAY_FLAG | GSS_C_SEQUENCE_FLAG |
+      GSS_C_INTEG_FLAG | GSS_C_CONF_FLAG |
+      GSS_C_PROT_READY_FLAG;
 
   if (initiator_cred_handle)
     {
@@ -189,56 +192,70 @@ gss_krb5_init_sec_context (OM_uint32 * minor_status,
 	 Later: if you have tickets for foo@BAR and bar@FOO, it may be
 	 useful to call gss_acquire_cred first to chose which one to
 	 initiate the context with.  Not many applications need this. */
-      if (minor_status)
-	*minor_status = 0;
       return GSS_S_NO_CRED;
     }
 
   if (k5 == NULL)
-    k5 = ctx->krb5 = xcalloc (sizeof (*k5), 1);
-
-  if (k5->sh == NULL)
     {
+      k5 = ctx->krb5 = xcalloc (sizeof (*k5), 1);
+
       rc = shishi_init (&k5->sh);
       if (rc != SHISHI_OK)
 	return GSS_S_FAILURE;
     }
 
-  if (ret_flags)
-    *ret_flags |= GSS_C_INTEG_FLAG | GSS_C_CONF_FLAG | GSS_C_SEQUENCE_FLAG;
-
   if (!k5->reqdone)
     {
-      return init_request (minor_status,
-			   initiator_cred_handle,
-			   context_handle,
-			   target_name,
-			   mech_type,
-			   req_flags,
-			   time_req,
-			   input_chan_bindings,
-			   input_token,
-			   actual_mech_type,
-			   output_token, ret_flags, time_rec);
-    }
-  else if (!k5->repdone)
-    {
-      return init_reply (minor_status,
-			 initiator_cred_handle,
-			 context_handle,
-			 target_name,
-			 mech_type,
-			 req_flags,
-			 time_req,
-			 input_chan_bindings,
-			 input_token,
-			 actual_mech_type,
-			 output_token, ret_flags, time_rec);
-    }
+      maj_stat = init_request (minor_status,
+			       initiator_cred_handle,
+			       context_handle,
+			       target_name,
+			       mech_type,
+			       req_flags,
+			       time_req,
+			       input_chan_bindings,
+			       input_token,
+			       actual_mech_type,
+			       output_token, ret_flags, time_rec);
+      if (GSS_ERROR (maj_stat))
+	return maj_stat;
 
-  if (minor_status)
-    *minor_status = 0;
-  return GSS_S_FAILURE;
+      k5->flags = req_flags & (/* GSS_C_DELEG_FLAG | */
+			       GSS_C_MUTUAL_FLAG |
+			       GSS_C_REPLAY_FLAG | GSS_C_SEQUENCE_FLAG |
+			       GSS_C_CONF_FLAG | GSS_C_INTEG_FLAG);
+
+      k5->key = shishi_ap_key (k5->ap);
+      k5->reqdone = 1;
+    }
+  else if (k5->reqdone && k5->flags & GSS_C_MUTUAL_FLAG && !k5->repdone)
+    {
+      maj_stat = init_reply (minor_status,
+			     initiator_cred_handle,
+			     context_handle,
+			     target_name,
+			     mech_type,
+			     req_flags,
+			     time_req,
+			     input_chan_bindings,
+			     input_token,
+			     actual_mech_type,
+			     output_token, ret_flags, time_rec);
+      if (GSS_ERROR (maj_stat))
+	return maj_stat;
+
+      if (ret_flags)
+	*ret_flags |= GSS_C_MUTUAL_FLAG;
+
+      k5->repdone = 1;
+    }
+  else
+    maj_stat = GSS_S_FAILURE;
+
+  if (time_rec)
+    *time_rec = gss_krb5_tktlifetime (k5->tkt);
+
+  return maj_stat;
 }
 
 /* Allows a remotely initiated security context between the
@@ -385,8 +402,8 @@ gss_krb5_accept_sec_context (OM_uint32 * minor_status,
   return GSS_S_COMPLETE;
 }
 
-/* Delete a krb5 security context.  Should only delete krb5 specific
-   part of context. */
+/* Delete a krb5 security context.  Assumes context_handle is valid.
+   Should only delete krb5 specific part of context. */
 OM_uint32
 gss_krb5_delete_sec_context (OM_uint32 * minor_status,
 			     gss_ctx_id_t * context_handle,
