@@ -127,8 +127,7 @@ inquire_cred (OM_uint32 * minor_status,
 	      const gss_cred_id_t cred_handle,
 	      gss_name_t * name,
 	      OM_uint32 * lifetime,
-	      gss_cred_usage_t * cred_usage,
-	      gss_OID_set * mechanisms)
+	      gss_cred_usage_t * cred_usage, gss_OID_set * mechanisms)
 {
   OM_uint32 maj_stat;
 
@@ -140,8 +139,7 @@ inquire_cred (OM_uint32 * minor_status,
       maj_stat = gss_create_empty_oid_set (minor_status, mechanisms);
       if (GSS_ERROR (maj_stat))
 	return maj_stat;
-      maj_stat = gss_add_oid_set_member (minor_status, GSS_KRB5,
-					 mechanisms);
+      maj_stat = gss_add_oid_set_member (minor_status, GSS_KRB5, mechanisms);
       if (GSS_ERROR (maj_stat))
 	return maj_stat;
     }
@@ -198,8 +196,7 @@ gss_krb5_inquire_cred_by_mech (OM_uint32 * minor_status,
 }
 
 OM_uint32
-gss_krb5_release_cred (OM_uint32 * minor_status,
-		       gss_cred_id_t * cred_handle)
+gss_krb5_release_cred (OM_uint32 * minor_status, gss_cred_id_t * cred_handle)
 {
   shishi_done ((*cred_handle)->krb5->sh);
   free ((*cred_handle)->krb5);
